@@ -25,6 +25,7 @@
 #include "interface.h"
 #include "main.h"
 #include "parser.h"
+#include "rar.h"
 #include "string_utils.h"
 #include "support.h"
 #include "window.h"
@@ -129,7 +130,8 @@ static gboolean is_encrypted (XArchive *archive)
 	return result;
 }
 
-gboolean is7zip_mhe (const gchar *filename)
+/* check for header encryption */
+gboolean is_7zip_mhe (const gchar *filename)
 {
 	GIOChannel *file;
 	guint i;
@@ -251,6 +253,7 @@ void xa_7zip_ask (XArchive *archive)
 	archive->can_delete = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive) && !READ_ONLY);
 	archive->can_sfx = (archive->type == XARCHIVETYPE_7ZIP);
 	archive->can_password = (archive->type == XARCHIVETYPE_7ZIP);
+	archive->can_encrypt = (archive->type == XARCHIVETYPE_7ZIP);
 	archive->can_full_path[0] = TRUE;
 	archive->can_overwrite = TRUE;
 	archive->can_update[1] = archiver[archive->type].is_compressor;
@@ -271,7 +274,7 @@ static gchar *xa_7zip_password_str (XArchive *archive)
 	if (archive->password)
 	{
 		escaped = xa_escape_bad_chars(archive->password, ESCAPES);
-		password_str = g_strconcat(" -p", escaped, NULL);
+		password_str = g_strconcat(" -p", escaped, archive->do_encrypt ? " -mhe" : "", NULL);
 		g_free(escaped);
 
 		return password_str;
@@ -294,6 +297,10 @@ static void xa_7zip_parse_output (gchar *line, XArchive *archive)
 
 	if (!data_line)
 	{
+		/* Since 7zip's plain list command does not indicate whether files are
+		 * encrypted, let's assume that 7zAES means all files are encrypted
+		 * (although some may be unencrypted).
+		 */
 		if (strncmp(line, "Method = ", 9) == 0 && strstr(line, "7zAES"))
 		{
 			encrypted = TRUE;
@@ -369,13 +376,10 @@ static void xa_7zip_parse_output (gchar *line, XArchive *archive)
 	LAST_ITEM(filename);
 	filename++;            // skip the additional spacing character
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		entry->is_encrypted = encrypted;
 
 		if (!entry->is_dir)
@@ -394,15 +398,28 @@ void xa_7zip_list (XArchive *archive)
 
 	if (!archive->has_password)
 	{
-		if (archive->type == XARCHIVETYPE_7ZIP)
-			archive->has_password = is7zip_mhe(archive->path[0]);
-		else
-			archive->has_password = is_encrypted(archive);
-	}
+		gboolean header_encryption = FALSE;
 
-	if ((archive->type == XARCHIVETYPE_7ZIP) && archive->has_password)
-		if (!xa_check_password(archive))
+		if (archive->type == XARCHIVETYPE_7ZIP)
+		{
+			archive->has_password = is_7zip_mhe(archive->path[0]);
+			header_encryption = archive->has_password;
+		}
+		else if ((archive->type == XARCHIVETYPE_RAR) && is_rar_hp(archive->path[0]))
+		{
+			archive->has_password = TRUE;
+			header_encryption = archive->has_password;
+		}
+		else
+			/* Since 7zip's plain list command does not indicate whether files are
+			 * encrypted, let's assume that, in archives of other types, either all
+			 * or none of the files are encrypted (although some may be unencrypted).
+			 */
+			archive->has_password = is_encrypted(archive);
+
+		if (header_encryption && !xa_check_password(archive))
 			return;
+	}
 
 	/* a single file compressor archive is no longer new and empty now */
 	archive->can_add = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive) && !READ_ONLY);
@@ -512,6 +529,12 @@ void xa_7zip_delete (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *password_str, *command;
 
+	/* necessary if there are still files in the archive */
+	/* wouldn't be necessary for the last file, though */
+	if (archive->type == XARCHIVETYPE_7ZIP)
+		if (archive->has_password && !xa_check_password(archive))
+			return;
+
 	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	password_str = xa_7zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0], " d", password_str, " -bd -spd -y ", archive->path[1], " --", files->str, NULL);
