@@ -190,27 +190,28 @@ static XEntry *xa_alloc_memory_for_each_row (guint columns, GType column_types[]
 	return entry;
 }
 
-static XEntry *xa_find_directory_entry (XEntry *entry, const gchar *name)
+static XEntry *xa_find_directory_entry (XEntry *entry, const gchar *name, gboolean same_enc)
 {
-	gchar *filename;
+	gchar *name_utf8 = NULL;
 
 	if (entry == NULL)
 		return NULL;
 
-	if (g_utf8_validate(entry->filename, -1, NULL))
-		filename = g_filename_display_name(name);
-	else
-		filename = g_strdup(name);
-
-	if (entry->is_dir && strcmp(entry->filename, filename) == 0)
+	if (entry->is_dir)
 	{
-		g_free(filename);
-		return entry;
+		if (!same_enc && g_utf8_validate(entry->filename, -1, NULL) && !g_utf8_validate(name, -1, NULL))
+			name_utf8 = g_filename_display_name(name);
+
+		if (strcmp(entry->filename, name_utf8 ? name_utf8 : name) == 0)
+		{
+			g_free(name_utf8);
+			return entry;
+		}
+
+		g_free(name_utf8);
 	}
 
-	g_free(filename);
-
-  return xa_find_directory_entry(entry->next, name);
+  return xa_find_directory_entry(entry->next, name, same_enc);
 }
 
 static gpointer *xa_fill_archive_entry_columns_for_each_row (XArchive *archive, XEntry *entry, gpointer *items)
@@ -415,7 +416,7 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 		g_error_free(error);
 
 		if (xa_main_window)
-			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->sorted, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
+			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
 
 		archive->status = XARCHIVESTATUS_ERROR;
 		return;
@@ -705,7 +706,7 @@ void xa_free_entry (XArchive *archive, XEntry *entry)
 	g_free(entry);
 }
 
-XEntry *xa_set_archive_entries_for_each_row (XArchive *archive, const gchar *filename, gpointer *items)
+XEntry *xa_set_archive_entries_for_each_row (XArchive *archive, const gchar *filename, gpointer *items, gboolean is_dir)
 {
 	XEntry *entry = NULL, *last = archive->root_entry;
 	gchar **components;
@@ -725,19 +726,20 @@ XEntry *xa_set_archive_entries_for_each_row (XArchive *archive, const gchar *fil
 
 	while (components[n] && *components[n])
 	{
-		entry = xa_find_directory_entry(last->child, components[n]);
+		if (components[n + 1] || is_dir)
+			entry = xa_find_directory_entry(last->child, components[n], TRUE);
+		else
+			entry = NULL;
 
 		if (entry == NULL)
 		{
 			entry = xa_alloc_memory_for_each_row(archive->columns, archive->column_types);
 
 			if (entry == NULL)
-				return NULL;
+				goto done;
 
 			entry->filename = g_strdup(components[n]);
-
-			if (components[n + 1])
-				entry->is_dir = TRUE;
+			entry->is_dir = (components[n + 1] || is_dir);
 
 			entry->next = last->child;
 			last->child = entry;
@@ -751,6 +753,7 @@ XEntry *xa_set_archive_entries_for_each_row (XArchive *archive, const gchar *fil
 		n++;
 	}
 
+done:
 	g_strfreev(components);
 
 	return entry;
@@ -776,7 +779,7 @@ XEntry* xa_find_entry_from_dirpath (XArchive *archive, const gchar *dirpath)
 
 	while (components[n] && *components[n])
 	{
-		entry = xa_find_directory_entry(root->child, components[n]);
+		entry = xa_find_directory_entry(root->child, components[n], FALSE);
 		root = entry;
 		n++;
 	}
