@@ -214,13 +214,10 @@ static void xa_arj_parse_output (gchar *line, XArchive *archive)
 			/* skip entry since unarj lacks directory structure information */
 			entry = NULL;
 		else
-			entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+			entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 		if (entry)
 		{
-			if (dir)
-				entry->is_dir = TRUE;
-
 			entry->is_encrypted	= encrypted;
 
 			if (!entry->is_dir)
