@@ -135,13 +135,10 @@ void xa_cpio_parse_output (gchar *line, XArchive *archive)
 		}
 	}
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		if (!entry->is_dir)
 			archive->files++;
 
