@@ -609,13 +609,13 @@ void xa_parse_extract_dialog_options (XArchive *archive, ExtractDialog *extract_
 
 			if (g_file_test (destination_path,G_FILE_TEST_IS_DIR)&& access (destination_path,R_OK | W_OK | X_OK ))
 			{
-				gchar *utf8_path;
+				gchar *path_utf8;
 				gchar  *msg;
 
-                utf8_path = g_filename_to_utf8 (destination_path,-1,NULL,NULL,NULL);
-                msg = g_strdup_printf (_("You don't have the right permissions to extract the files to the directory \"%s\"."),utf8_path);
+                path_utf8 = g_filename_display_name(destination_path);
+                msg = g_strdup_printf (_("You don't have the right permissions to extract the files to the directory \"%s\"."),path_utf8);
                 xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't perform extraction!"),msg );
-                g_free (utf8_path);
+                g_free (path_utf8);
 				g_free (msg);
 				g_free (destination_path);
 				break;
@@ -688,7 +688,7 @@ void xa_parse_extract_dialog_options (XArchive *archive, ExtractDialog *extract_
 			else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(extract_dialog->selected_files)))
 				gtk_tree_selection_selected_foreach(selection,(GtkTreeSelectionForeachFunc)xa_concat_selected_filenames,&names);
 			if (xa_main_window)
-				xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
+				xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
 
 			xa_execute_extract_commands(archive, names, strip);
 		}
@@ -698,7 +698,6 @@ void xa_parse_extract_dialog_options (XArchive *archive, ExtractDialog *extract_
 
 MultiExtractDialog *xa_create_multi_extract_dialog ()
 {
-	GTK_COMPAT_TOOLTIPS;
 	MultiExtractDialog *multi_extract;
 	GtkWidget *vbox, *window, *alignment, *hbox, *hbox2, *button, *frame, *image, *label;
 	GtkCellRenderer *renderer;
