@@ -18,6 +18,7 @@
  *  MA 02110-1301 USA.
  */
 
+#include <signal.h>
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
 #include "interface.h"
@@ -26,7 +27,6 @@
 #include "pref_dialog.h"
 #include "rar.h"
 #include "string_utils.h"
-#include "support.h"
 #include "window.h"
 #include "zip.h"
 
@@ -69,7 +69,7 @@ GtkWidget *toolbar1;
 GtkWidget *toolbar2;
 GtkWidget *total_label;
 GtkWidget *type_data;
-GtkWidget *unsort_menu;
+GtkWidget *sort_menu;
 GtkWidget *up_button;
 GtkWidget *view;
 GtkWidget *view_shell_output1;
@@ -101,12 +101,14 @@ static gulong changed;
 static const GtkTargetEntry drag_targets[] =
 {
 	{"XdndDirectSave0", 0, TARGET_TYPE_DIRECT_SAVE},
-	{"text/uri-list", 0, TARGET_TYPE_URI_LIST}
+	{"xa-internal/uri-list", 0, TARGET_TYPE_XA_INTERNAL},   // identical to text/uri-list, but necessary
+	{"text/uri-list", 0, TARGET_TYPE_URI_LIST}              // because text/uri-list may be disabled
 };
 
 static const GtkTargetEntry drop_targets[] =
 {
-	{"text/uri-list", 0, 0}
+	{"xa-internal/uri-list", 0, TARGET_TYPE_XA_INTERNAL},
+	{"text/uri-list", 0, TARGET_TYPE_URI_LIST}
 };
 
 static void xa_create_popup_menu ()
@@ -311,7 +313,7 @@ failed:
 
 	/* add dragged files inside the determined archive location path */
 	g_free(archive[idx]->location_path);
-	archive[idx]->location_path = g_strdup(pathname->str);
+	archive[idx]->location_path = g_filename_from_utf8(pathname->str, -1, NULL, NULL, NULL);
 
 	archive[idx]->child_dir = g_path_get_dirname(list->data);
 
@@ -339,7 +341,7 @@ failed:
 	gtk_drag_finish(context, TRUE, FALSE, time);
 }
 
-static gboolean xa_dir_sidebar_drag_motion_expand (GdkDragContext *context)
+static gboolean xa_dir_sidebar_drag_motion_expand (GtkWidget *widget)
 {
 	GtkTreePath *path;
 
@@ -351,9 +353,7 @@ static gboolean xa_dir_sidebar_drag_motion_expand (GdkDragContext *context)
 		gtk_tree_path_free(path);
 	}
 
-	/* if context still valid */
-	if (G_IS_OBJECT(context))
-		g_object_steal_data(G_OBJECT(context), "expand_row");
+	g_object_steal_data(G_OBJECT(widget), "expand_row");
 
 	return FALSE;
 }
@@ -366,10 +366,10 @@ static gboolean xa_dir_sidebar_drag_motion (GtkWidget *widget, GdkDragContext *c
 
 	if (path)
 	{
-		if (!g_object_get_data(G_OBJECT(context), "expand_row"))
+		if (!g_object_get_data(G_OBJECT(widget), "expand_row"))
 		{
-			g_timeout_add_full(G_PRIORITY_LOW, 1000, (GSourceFunc) xa_dir_sidebar_drag_motion_expand, context, NULL);
-			g_object_set_data(G_OBJECT(context), "expand_row", GINT_TO_POINTER(TRUE));
+			g_timeout_add_full(G_PRIORITY_LOW, 1000, (GSourceFunc) xa_dir_sidebar_drag_motion_expand, widget, NULL);
+			g_object_set_data(G_OBJECT(widget), "expand_row", GINT_TO_POINTER(TRUE));
 		}
 
 		g_object_set_data_full(G_OBJECT(context), "tree_path", path, (GDestroyNotify) gtk_tree_path_free);
@@ -465,6 +465,7 @@ static void xa_page_has_changed (GtkNotebook *notebook, GTK_COMPAT_SWITCH_PAGE_T
 
 	xa_set_window_title(xa_main_window, archive[idx]->path[0]);
 	xa_restore_navigation(idx);
+	gtk_menu_item_set_label(GTK_MENU_ITEM(sort_menu), xa_label_sort(archive[idx]));
 	xa_set_statusbar_message_for_displayed_rows(archive[idx]);
 
 	if (archive[idx]->treeview != NULL)
@@ -501,7 +502,7 @@ static void xa_page_has_changed (GtkNotebook *notebook, GTK_COMPAT_SWITCH_PAGE_T
 
 		xa_block_signal_dir_treeview_selection(FALSE);
 	}
-	xa_set_button_state(1, 1, 1, 1, archive[idx]->can_test, 1, archive[idx]->can_add, archive[idx]->can_extract, archive[idx]->sorted, archive[idx]->can_sfx, archive[idx]->has_comment, archive[idx]->output, archive[idx]->has_password);
+	xa_set_button_state(1, 1, 1, 1, archive[idx]->can_test, 1, archive[idx]->can_add, archive[idx]->can_extract, archive[idx]->can_sfx, archive[idx]->has_comment, archive[idx]->output, archive[idx]->has_password);
 }
 
 static void xa_select_by_pattern_dialog (GtkMenuItem *menuitem, gpointer user_data)
@@ -737,6 +738,18 @@ static void xa_icon_theme_changed (GtkIconTheme *icon_theme, gpointer data)
  * time as the filenames currently displayed. What of the other tabs then?
 }*/
 
+gchar *xa_label_sort (XArchive *archive)
+{
+	gboolean sorted;
+
+	if (archive)
+		sorted = archive->sorted;
+	else
+		sorted = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->sort_by_filenames));
+
+	return sorted ? _("Uns_ort") : _("S_ort");
+}
+
 void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, gboolean show_sidebar, gboolean show_toolbar)
 {
 	GTK_COMPAT_TOOLTIPS;
@@ -895,14 +908,14 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_widget_show (tmp_image);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (rename_menu),tmp_image);
 
-	unsort_menu = gtk_image_menu_item_new_with_mnemonic(_("Uns_ort"));
-	gtk_widget_set_sensitive(unsort_menu, FALSE);
-	gtk_widget_show(unsort_menu);
-	gtk_container_add(GTK_CONTAINER(menuitem2_menu), unsort_menu);
+	sort_menu = gtk_image_menu_item_new_with_mnemonic(xa_label_sort(NULL));
+	gtk_widget_set_sensitive(sort_menu, FALSE);
+	gtk_widget_show(sort_menu);
+	gtk_container_add(GTK_CONTAINER(menuitem2_menu), sort_menu);
 
 	tmp_image = gtk_image_new_from_stock("gtk-undo", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(tmp_image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(unsort_menu), tmp_image);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sort_menu), tmp_image);
 
 	separatormenuitem3 = gtk_separator_menu_item_new ();
 	gtk_widget_show (separatormenuitem3);
@@ -1192,6 +1205,16 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_widget_show (hbox_sb);
 	gtk_box_pack_end (GTK_BOX (vbox1),hbox_sb,FALSE,TRUE,0);
 
+	green_led = xa_main_window_find_image("xarchiver-green.png", GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show(green_led);
+	gtk_box_pack_start(GTK_BOX(hbox_sb), green_led, FALSE, FALSE, 0);
+	gtk_misc_set_alignment(GTK_MISC(green_led), 1, 1);
+	gtk_widget_set_tooltip_text(green_led, _("This is Xarchiver's LED status. When it's flashing Xarchiver is busy."));
+
+	red_led = xa_main_window_find_image("xarchiver-red.png", GTK_ICON_SIZE_BUTTON);
+	gtk_box_pack_start(GTK_BOX(hbox_sb), red_led, FALSE, FALSE, 0);
+	gtk_misc_set_alignment(GTK_MISC(red_led), 1, 1);
+
 	total_frame = gtk_frame_new (NULL);
 	gtk_widget_show (total_frame);
 	gtk_box_pack_start (GTK_BOX (hbox_sb),total_frame,TRUE,TRUE,0);
@@ -1200,6 +1223,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 
 	total_label = gtk_label_new (NULL);
 	gtk_misc_set_alignment(GTK_MISC(total_label), 0, 0.5);
+	gtk_misc_set_padding(GTK_MISC(total_label), 1, 0);
 	gtk_widget_show (total_label);
 	gtk_container_add (GTK_CONTAINER (total_frame),total_label);
 
@@ -1210,19 +1234,10 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 
 	selected_label = gtk_label_new (NULL);
 	gtk_misc_set_alignment(GTK_MISC(selected_label), 0, 0.5);
+	gtk_misc_set_padding(GTK_MISC(selected_label), 1, 0);
 	gtk_widget_show (selected_label);
 	gtk_container_add (GTK_CONTAINER (selected_frame),selected_label);
 
-	green_led = xa_main_window_find_image("xarchiver-green.png", GTK_ICON_SIZE_BUTTON);
-	gtk_widget_show (green_led);
-	gtk_box_pack_start (GTK_BOX (hbox_sb),green_led,FALSE,FALSE,0);
-	gtk_misc_set_alignment (GTK_MISC (green_led),1,1);
-	gtk_widget_set_tooltip_text(green_led, _("This is Xarchiver's LED status. When it's flashing Xarchiver is busy."));
-
-	red_led = xa_main_window_find_image("xarchiver-red.png", GTK_ICON_SIZE_BUTTON);
-	gtk_box_pack_start (GTK_BOX (hbox_sb),red_led,FALSE,FALSE,0);
-	gtk_misc_set_alignment (GTK_MISC (red_led),1,1);
-
 	g_signal_connect(new1, "activate", G_CALLBACK(xa_new_archive), NULL);
 	g_signal_connect(open1, "activate", G_CALLBACK(xa_open_archive), NULL);
 	g_signal_connect(listing_text, "activate", G_CALLBACK(xa_list_archive), GUINT_TO_POINTER(0));
@@ -1247,7 +1262,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	g_signal_connect(multi_extract_menu, "activate", G_CALLBACK(xa_show_multi_extract_dialog), NULL);
 	g_signal_connect(help1, "activate", G_CALLBACK(xa_show_help), NULL);
 	g_signal_connect(about1, "activate", G_CALLBACK(xa_about), NULL);
-	g_signal_connect(unsort_menu, "activate", G_CALLBACK(xa_unsort), NULL);
+	g_signal_connect(sort_menu, "activate", G_CALLBACK(xa_sort_unsort), NULL);
 
 	g_signal_connect(New_button, "clicked", G_CALLBACK(xa_new_archive), NULL);
 	g_signal_connect(Open_button, "clicked", G_CALLBACK(xa_open_archive), NULL);
@@ -1368,7 +1383,7 @@ void xa_add_page (XArchive *archive)
 	gtk_tree_selection_set_mode(sel,GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rubber_banding(GTK_TREE_VIEW(archive->treeview),TRUE);
 
-	gtk_drag_source_set(archive->treeview, GDK_BUTTON1_MASK, drag_targets, G_N_ELEMENTS(drag_targets), GDK_ACTION_COPY);
+	gtk_drag_source_set(archive->treeview, GDK_BUTTON1_MASK, drag_targets, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->extended_dnd)) ? G_N_ELEMENTS(drag_targets) : 2, GDK_ACTION_COPY);
 	g_signal_connect(sel, "changed", G_CALLBACK(xa_row_selected), archive);
 	g_signal_connect(G_OBJECT(archive->treeview), "drag-begin", G_CALLBACK(xa_treeview_drag_begin), archive);
 	g_signal_connect(G_OBJECT(archive->treeview), "drag-data-get", G_CALLBACK(xa_treeview_drag_data_get), archive);
@@ -1657,7 +1672,7 @@ GtkWidget *xa_create_archive_properties_window()
 	return archive_properties_window;
 }
 
-void xa_set_button_state (gboolean new, gboolean open, gboolean list, gboolean save, gboolean test, gboolean close, gboolean add, gboolean extract, gboolean sorted, gboolean sfx, gboolean comment, GSList *output, gboolean password)
+void xa_set_button_state (gboolean new, gboolean open, gboolean list, gboolean save, gboolean test, gboolean close, gboolean add, gboolean extract, gboolean sfx, gboolean comment, GSList *output, gboolean password)
 {
 	gtk_widget_set_sensitive(new1, new);
 	gtk_widget_set_sensitive(New_button, new);
@@ -1672,7 +1687,7 @@ void xa_set_button_state (gboolean new, gboolean open, gboolean list, gboolean s
 	gtk_widget_set_sensitive(AddFile_button, add);
 	gtk_widget_set_sensitive(extract_menu, extract);
 	gtk_widget_set_sensitive(Extract_button, extract);
-	gtk_widget_set_sensitive(unsort_menu, sorted && close);
+	gtk_widget_set_sensitive(sort_menu, close);
 	gtk_widget_set_sensitive(exe_menu, sfx);
 	gtk_widget_set_sensitive(comment_menu, comment);
 	gtk_widget_set_sensitive(select_all, close);
@@ -1728,7 +1743,8 @@ void xa_show_progress_bar (XArchive *archive)
 	pixbuf = gtk_icon_theme_load_icon(icon_theme, "gnome-mime-application-zip", 40, (GtkIconLookupFlags) 0, NULL);
 	if (!pixbuf) pixbuf = gtk_icon_theme_load_icon(icon_theme,"package-x-generic",40,GTK_ICON_LOOKUP_FORCE_SIZE,NULL);
 	icon_pixbuf = gtk_image_new_from_pixbuf(pixbuf);
-	g_object_unref(pixbuf);
+	if (pixbuf)
+		g_object_unref(pixbuf);
 
 	gtk_box_pack_start (GTK_BOX (hbox1),icon_pixbuf,FALSE,FALSE,0);
 	gtk_misc_set_alignment(GTK_MISC(icon_pixbuf), 0, 0);
