@@ -23,10 +23,12 @@
 
 #include <gtk/gtk.h>
 #include "archive.h"
+#include "support.h"
 
 enum
 {
   TARGET_TYPE_DIRECT_SAVE,
+  TARGET_TYPE_XA_INTERNAL,
   TARGET_TYPE_URI_LIST
 };
 
@@ -78,7 +80,7 @@ extern GtkWidget *toolbar1;
 extern GtkWidget *toolbar2;
 extern GtkWidget *total_label;
 extern GtkWidget *type_data;
-extern GtkWidget *unsort_menu;
+extern GtkWidget *sort_menu;
 extern GtkWidget *up_button;
 extern GtkWidget *view;
 extern GtkWidget *view_shell_output1;
@@ -100,8 +102,9 @@ void xa_disable_delete_buttons(gboolean);
 GSList *xa_file_filter_add_archiver_pattern_sort(GtkFileFilter *);
 gboolean xa_flash_led_indicator(XArchive *);
 void xa_increase_progress_bar(Progress *, gchar *, double);
+gchar *xa_label_sort(XArchive *);
 gboolean xa_pulse_progress_bar(XArchive *);
-void xa_set_button_state(gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, GSList *, gboolean);
+void xa_set_button_state(gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, gboolean, GSList *, gboolean);
 void xa_set_location_entry(XArchive *);
 void xa_show_progress_bar(XArchive *);
 
