@@ -69,7 +69,7 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 {
 	XEntry *entry;
 	gpointer item[7];
-	gchar *filename, time[6];
+	gchar *filename, permissions[11], time[6];
 	gboolean dir, link;
 
 	USE_PARSER;
@@ -94,14 +94,30 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 		return;
 	}
 
-	/* permissions */
-	NEXT_ITEM(item[5]);
+	/* permissions (may contain spaces) */
 
-	dir = (*(char *) item[5] == 'd');
-	link = (*(char *) item[5] == 'l');
+	item[5] = strncpy(permissions, line, sizeof(permissions));
+	permissions[sizeof(permissions) -1] = 0;
+	g_strchomp(permissions);
 
-	/* uid/gid */
-	NEXT_ITEM(item[6]);
+	dir = (*permissions == 'd');
+	link = (*permissions == 'l');
+
+	LINE_SKIP(sizeof(permissions));
+
+	/* check for uid/gid (may be empty) */
+
+	LINE_PEEK(5);
+
+	if (line[peek] == '/')
+	{
+		NEXT_ITEM(item[6]);
+
+		if (*(char *) item[6] == '*')   // lhasa, since v0.5.0
+			item[6] = NULL;
+	}
+	else
+		item[6] = NULL;
 
 	/* size */
 	NEXT_ITEM(item[1]);
@@ -111,20 +127,39 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 
 	LINE_PEEK(9);
 
-	/* date and time */
-	NEXT_ITEMS(3, item[3]);
+	/* date and time (may be empty) */
+
+	if (line[peek] != ' ')
+		NEXT_ITEMS(3, item[3]);
+	else
+	{
+		item[3] = NULL;
+		item[4] = NULL;
+		LINE_SKIP(13);
+	}
 
 	/* time */
-	if (((char *) item[3])[peek] == ':' && strlen(item[3]) >= 12)
+	if (item[3])
 	{
-		memcpy(time, item[3] + 7, 5);
-		time[5] = 0;
-	}
-	else
-		strcpy(time, "-----");
+		if (((char *) item[3])[peek] == ':' && strlen(item[3]) >= 12)
+		{
+			memcpy(time, item[3] + 7, 5);
+			time[5] = 0;
+		}
+		else
+			strcpy(time, "-----");
 
-	item[3] = date_MMM_dD_HourYear(item[3]);
-	item[4] = time;
+		if (((char *) item[3])[peek] == '*')   // lhasa, since v0.5.0
+		{
+			item[3] = NULL;
+			item[4] = NULL;
+		}
+		else
+		{
+			item[3] = date_MMM_dD_HourYear(item[3]);
+			item[4] = time;
+		}
+	}
 
 	/* name */
 	LAST_ITEM(filename);
@@ -142,13 +177,10 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 		}
 	}
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		if (!entry->is_dir)
 			archive->files++;
 
