@@ -626,8 +626,10 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "lz4");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz4");
 		/* Mozilla's proprietary LZ4 Frame Formats */
+		archiver[type].type = g_slist_append(archiver[type].type, "mozlz4");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.mozlz4");
 		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('m'));
-		archiver[type].tags = g_slist_append(archiver[type].tags, "mozlz4");
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG('m','j')));
 		archiver[type].tags = g_slist_append(archiver[type].tags, "mozJSSClz4");
 
@@ -897,6 +899,9 @@ static void xa_check_available_archivers ()
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.sfs");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.sqfs");
 
+		/* AppImage */
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('a'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, "AppImage");
 		/* snap package */
 		archiver[type].type = g_slist_append(archiver[type].type, "snap");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.snap");
@@ -1225,7 +1230,7 @@ static XArchive *xa_init_structure_from_cmd_line (char *filename)
 	{
 		gchar *name, *msg;
 
-		name = g_path_get_basename(filename);
+		name = g_filename_display_basename(filename);
 		msg = g_strdup_printf(_("The proper archiver for \"%s\" is not installed!"), name);
 		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), msg);
 		g_free(msg);
@@ -1345,11 +1350,8 @@ int main (int argc, char **argv)
 			if (archive->status == XARCHIVESTATUS_ERROR)
 				goto leave;
 
-			if (archive->has_password)
-			{
-				if (!xa_check_password(archive))
-					goto leave;
-			}
+			if (archive->has_password && !xa_check_password(archive))
+				goto leave;
 
 			if (opt_ensure_dir)
 			{
@@ -1402,11 +1404,8 @@ int main (int argc, char **argv)
 			if (archive->status == XARCHIVESTATUS_ERROR)
 				goto leave;
 
-			if (archive->has_password)
-			{
-				if (!xa_check_password(archive))
-					goto leave;
-			}
+			if (archive->has_password && !xa_check_password(archive))
+				goto leave;
 
 			xa_set_extract_dialog_options(extract_window,0,archive);
 			xa_parse_extract_dialog_options (archive,extract_window,NULL);
@@ -1499,11 +1498,8 @@ int main (int argc, char **argv)
 			if (archive->status == XARCHIVESTATUS_ERROR)
 				goto leave;
 
-			if (archive->has_password)
-			{
-				if (!xa_check_password(archive))
-					goto leave;
-			}
+			if (archive->has_password && !xa_check_password(archive))
+				goto leave;
 
 			xa_set_add_dialog_options(add_window, archive);
 			xa_parse_add_dialog_options(archive, add_window, NULL);
