@@ -25,10 +25,10 @@
 #include "archive.h"
 #include "gzip_et_al.h"
 
-#define COPYRIGHT_YEAR "2005-2014"
+#define COPYRIGHT_YEAR "2005–2014"
 #define COPYRIGHT_HOLDER "Giuseppe Torelli"
 #define MAINTAINER "Ingo Brückl"
-#define MAINTAINER_YEAR "2015-2024"
+#define MAINTAINER_YEAR "2015–2025"
 
 extern GtkWidget *xa_main_window;
 
