@@ -100,6 +100,7 @@ static void xa_prefs_dialog_set_default_options (PrefsDialog *prefs_dialog)
 	gtk_combo_box_set_active(GTK_COMBO_BOX(prefs_dialog->preferred_extract_dir), 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dialog->save_geometry), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dialog->allow_sub_dir), FALSE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dialog->extended_dnd), g_getenv("WAYLAND_DISPLAY") != NULL);
 	/* Set the default options in the extract dialog */
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory), TRUE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->full_path), TRUE);
@@ -296,7 +297,7 @@ PrefsDialog *xa_create_prefs_dialog ()
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_notebook_append_page(GTK_NOTEBOOK(prefs_dialog->notebook), vbox, NULL);
 
-	table = gtk_table_new(9, 2, FALSE);
+	table = gtk_table_new(10, 2, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 1);
 
@@ -413,6 +414,12 @@ PrefsDialog *xa_create_prefs_dialog ()
 	                 0, 2, 8, 9, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 	gtk_button_set_focus_on_click(GTK_BUTTON(prefs_dialog->allow_sub_dir), FALSE);
 
+	prefs_dialog->extended_dnd = gtk_check_button_new_with_mnemonic(_("Extended drag-and-drop support (requires restart)"));
+	gtk_widget_set_tooltip_text(prefs_dialog->extended_dnd, _("This enables drag-and-drop transfers of type \"text/uri-list\", which is only necessary if you are not using the X Window System or if the targets do not support XDirectSave.\nIf you are using the X Window System and drag-and-drop works without this extension, do not enable it to avoid drawbacks."));
+	gtk_table_attach(GTK_TABLE(table), prefs_dialog->extended_dnd,
+	                 0, 2, 9, 10, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 4);
+	gtk_button_set_focus_on_click(GTK_BUTTON(prefs_dialog->extended_dnd), FALSE);
+
 	return prefs_dialog;
 }
 
@@ -493,6 +500,7 @@ void xa_prefs_save_options (PrefsDialog *prefs_dialog, const char *filename)
 		g_free(value);
 	}
 	g_key_file_set_integer(xa_key_file, PACKAGE, "allow_sub_dir", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_dialog->allow_sub_dir)));
+	g_key_file_set_integer(xa_key_file, PACKAGE, "extended_dnd", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_dialog->extended_dnd)));
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_dialog->save_geometry)))
 	{
 		/* Main window coords */
@@ -550,7 +558,7 @@ void xa_prefs_load_options (PrefsDialog *prefs_dialog)
 	gchar *xarchiver_config_dir, *old_dir, *old_config;
 	GKeyFile *xa_key_file = g_key_file_new();
 	GError *error = NULL;
-	gboolean unzip, aisearch, aexpand, toolbar;
+	gboolean unzip, aisearch, aexpand, toolbar, extdnd;
 	gint idx, val;
 
 	config_dir = g_get_user_config_dir();
@@ -655,6 +663,17 @@ void xa_prefs_load_options (PrefsDialog *prefs_dialog)
 		}
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dialog->show_toolbar), toolbar);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dialog->allow_sub_dir), g_key_file_get_boolean(xa_key_file, PACKAGE, "allow_sub_dir", NULL));
+		extdnd = g_key_file_get_boolean(xa_key_file, PACKAGE, "extended_dnd", &error);
+		if (error)
+		{
+			if (error->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
+				/* default setting with existing, old config file */
+				extdnd = (g_getenv("WAYLAND_DISPLAY") != NULL);
+
+			g_error_free(error);
+			error = NULL;
+		}
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_dialog->extended_dnd), extdnd);
 		if (!xdg_open)
 		{
 			value = g_key_file_get_string(xa_key_file,PACKAGE,"preferred_web_browser",NULL);
