@@ -58,6 +58,62 @@ void xa_rar_check_version (gchar *path)
 	g_free(output);
 }
 
+/* check for header encryption */
+gboolean is_rar_hp (const gchar *filename)
+{
+	GIOChannel *file;
+	gboolean result = FALSE;
+
+	file = g_io_channel_new_file(filename, "r", NULL);
+
+	if (file)
+	{
+		gchar byte[2];
+
+		g_io_channel_set_encoding(file, NULL, NULL);
+
+		/* skip RAR 4 and 5 common signature part */
+		g_io_channel_seek_position(file, 6, G_SEEK_SET, NULL);
+
+		g_io_channel_read_chars(file, byte, sizeof(*byte), NULL, NULL);
+
+		/* RAR 4 archive */
+		if (*byte == 0)
+		{
+			/* skip header CRC16 */
+			g_io_channel_seek_position(file, 2, G_SEEK_CUR, NULL);
+
+			/* block type */
+			g_io_channel_read_chars(file, &byte[0], sizeof(*byte), NULL, NULL);
+
+			/* block flag */
+			g_io_channel_read_chars(file, &byte[1], sizeof(*byte), NULL, NULL);
+
+			result = (byte[0] == 0x73 && byte[1] & 0x80);
+		}
+		/* RAR 5 archive */
+		else
+		{
+			/* skip last signature byte and header CRC32 */
+			g_io_channel_seek_position(file, 5, G_SEEK_CUR, NULL);
+
+			/* skip vint header size */
+			do
+				g_io_channel_read_chars(file, byte, sizeof(*byte), NULL, NULL);
+			while (*byte & 0x80);
+
+			/* header type */
+			g_io_channel_read_chars(file, byte, sizeof(*byte), NULL, NULL);
+
+			result = (*byte == 4);
+		}
+
+		g_io_channel_shutdown(file, FALSE, NULL);
+	}
+
+	return result;
+}
+
 void xa_rar_ask (XArchive *archive)
 {
 	gboolean read_only;
@@ -71,6 +127,7 @@ void xa_rar_ask (XArchive *archive)
 	archive->can_delete = (archiver[archive->type].is_compressor && !read_only);
 	archive->can_sfx = archiver[archive->type].is_compressor;
 	archive->can_password = archiver[archive->type].is_compressor;
+	archive->can_encrypt = archiver[archive->type].is_compressor;
 	archive->can_full_path[0] = TRUE;
 	archive->can_full_path[1] = archiver[archive->type].is_compressor;
 	archive->can_touch = TRUE;
@@ -95,7 +152,7 @@ static gchar *xa_rar_password_str (XArchive *archive)
 	if (archive->password)
 	{
 		escaped = xa_escape_bad_chars(archive->password, ESCAPES);
-		password_str = g_strconcat(" -p", escaped, NULL);
+		password_str = g_strconcat(" -", archive->do_encrypt ? "h" : "", "p", escaped, NULL);
 		g_free(escaped);
 
 		return password_str;
@@ -218,13 +275,10 @@ static void xa_rar_v4_parse_output (gchar *line, XArchive *archive)
 		/* version */
 		NEXT_ITEM(item[8]);
 
-		entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+		entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 		if (entry)
 		{
-			if (dir)
-				entry->is_dir = TRUE;
-
 			entry->is_encrypted = encrypted;
 
 			if (!entry->is_dir)
@@ -336,7 +390,20 @@ static void xa_rar_parse_output (gchar *line, XArchive *archive)
 	NEXT_ITEM(item[4]);
 
 	/* checksum */
-	NEXT_ITEM(item[6]);
+
+	LINE_PEEK(8);
+
+	if (line[peek] == ' ')
+	{
+		line += peek + 1;
+
+		if (*line)
+			*line++ = 0;
+
+		item[6] = NULL;
+	}
+	else
+		NEXT_ITEM(item[6]);
 
 	/* name (follows with two characters spacing instead of one) */
 	LAST_ITEM(filename);
@@ -354,13 +421,10 @@ static void xa_rar_parse_output (gchar *line, XArchive *archive)
 		}
 	}
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		entry->is_encrypted = encrypted;
 
 		if (!entry->is_dir)
@@ -372,55 +436,12 @@ static void xa_rar_parse_output (gchar *line, XArchive *archive)
 
 void xa_rar_list (XArchive *archive)
 {
-	GIOChannel *file;
 	gchar *password_str, *command;
 	guint i;
 
-	file = g_io_channel_new_file(archive->path[0], "r", NULL);
-
-	if (file)
+	if (!archive->has_password)
 	{
-		gchar byte[2];
-
-		g_io_channel_set_encoding(file, NULL, NULL);
-
-		/* skip RAR 4 and 5 common signature part */
-		g_io_channel_seek_position(file, 6, G_SEEK_SET, NULL);
-
-		g_io_channel_read_chars(file, byte, sizeof(*byte), NULL, NULL);
-
-		/* RAR 4 archive */
-		if (*byte == 0)
-		{
-			/* skip header CRC16 */
-			g_io_channel_seek_position(file, 2, G_SEEK_CUR, NULL);
-
-			/* block type */
-			g_io_channel_read_chars(file, &byte[0], sizeof(*byte), NULL, NULL);
-
-			/* block flag */
-			g_io_channel_read_chars(file, &byte[1], sizeof(*byte), NULL, NULL);
-
-			archive->has_password = (byte[0] == 0x73 && byte[1] & 0x80);
-		}
-		/* RAR 5 archive */
-		else
-		{
-			/* skip last signature byte and header CRC32 */
-			g_io_channel_seek_position(file, 5, G_SEEK_CUR, NULL);
-
-			/* skip vint header size */
-			do
-				g_io_channel_read_chars(file, byte, sizeof(*byte), NULL, NULL);
-			while (*byte & 0x80);
-
-			/* header type */
-			g_io_channel_read_chars(file, byte, sizeof(*byte), NULL, NULL);
-
-			archive->has_password = (*byte == 4);
-		}
-
-		g_io_channel_shutdown(file, FALSE, NULL);
+		archive->has_password = is_rar_hp(archive->path[0]);
 
 		if (archive->has_password && !xa_check_password(archive))
 			return;
@@ -556,10 +577,12 @@ void xa_rar_add (XArchive *archive, GSList *file_list)
 void xa_rar_delete (XArchive *archive, GSList *file_list)
 {
 	GString *files;
-	gchar *command;
+	gchar *password_str, *command;
 
 	files = xa_quote_filenames(file_list, NULL, DIR_WITHOUT_SLASH);
-	command = g_strconcat(archiver[archive->type].program[0], " d -idp -y ", archive->path[1], " --", files->str, NULL);
+	password_str = xa_rar_password_str(archive);
+	command = g_strconcat(archiver[archive->type].program[0], " d", password_str, " -idp -y ", archive->path[1], " --", files->str, NULL);
+	g_free(password_str);
 	g_string_free(files,TRUE);
 
 	xa_run_command(archive, command);
