@@ -139,13 +139,10 @@ static void xa_squashfs_parse_output (gchar *line, XArchive *archive)
 	if (strncmp(filename, "squashfs-root/", 14) == 0)
 		filename += 14;
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		if (!entry->is_dir)
 			archive->files++;
 
