@@ -46,6 +46,10 @@
 static inline int g_strcmp0 (const char *str1, const char *str2) { if (!str1) return -(str1 != str2); if (!str2) return (str1 != str2); return strcmp(str1, str2); }
 #endif
 
+#if !GLIB_CHECK_VERSION(2,25,0)
+#define GStatBuf struct stat
+#endif
+
 #if !GLIB_CHECK_VERSION(2,28,0)
 #define g_slist_free_full(list, free_func) do { g_slist_foreach(list, (GFunc) free_func, NULL); g_slist_free(list); } while (0)
 #endif
@@ -88,6 +92,11 @@ static inline void GTK_COMPAT_ENTRY_ICON (GtkWidget *entry, gpointer callback, g
 {
 	sexy_icon_entry_add_clear_button(SEXY_ICON_ENTRY(entry), user_data, callback);
 }
+static inline void gtk_menu_item_set_label (GtkMenuItem *menu_item, const gchar *label)
+{
+	if (GTK_IS_LABEL(GTK_BIN(menu_item)->child))
+		gtk_label_set_label(GTK_LABEL(GTK_BIN(menu_item)->child), label ? label : "");
+}
 #else
 #define GTK_COMPAT_ENTRY_ICON_TYPE GdkEvent *
 #define GTK_COMPAT_ENTRY_ICON_NEW gtk_entry_new
@@ -146,10 +155,11 @@ static inline void gtk_icon_view_set_item_orientation (GtkIconView *icon_view, G
 #endif
 
 #if !GTK_CHECK_VERSION(2,24,0)
-#define GDK_COMPAT_SPAWN(p1, p2, p3, p4, p5, p6, p7, p8) gdk_spawn_on_screen(screen, p1, p2, p3, p4, NULL, NULL, p7, p8)
+#define GDK_COMPAT_SPAWN(p1, p2, p3, p4, p5, p6, p7, p8) gdk_spawn_on_screen(gdk_screen_get_default(), p1, p2, p3, p4, NULL, NULL, p7, p8)
 #define GTK_COMPAT_ABOUT_DIALOG_URI(about, func) \
 	gtk_about_dialog_set_email_hook(func, NULL, NULL); \
 	gtk_about_dialog_set_url_hook(func, NULL, NULL)
+#define GtkComboBoxText GtkWidget
 #define GTK_COMBO_BOX_TEXT
 #define gtk_combo_box_text_new gtk_combo_box_new_text
 static inline void gtk_combo_box_text_append_text (GtkWidget *combo_box, const gchar *text)
