@@ -23,6 +23,7 @@
 #include "interface.h"
 #include "main.h"
 #include "parser.h"
+#include "rar.h"
 #include "string_utils.h"
 #include "support.h"
 #include "window.h"
@@ -149,13 +150,10 @@ static void xa_unar_parse_output (gchar *line, XArchive *archive)
 		}
 	}
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		entry->is_encrypted = encrypted;
 
 		if (!entry->is_dir)
@@ -172,10 +170,12 @@ void xa_unar_list (XArchive *archive)
 	gchar *password_str, *command;
 	guint i;
 
-	if (archive->type == XARCHIVETYPE_7ZIP)
+	if (!archive->has_password)
 	{
-		if (!archive->has_password)
-			archive->has_password = is7zip_mhe(archive->path[0]);
+		if (archive->type == XARCHIVETYPE_7ZIP)
+			archive->has_password = is_7zip_mhe(archive->path[0]);
+		else if (archive->type == XARCHIVETYPE_RAR)
+			archive->has_password = is_rar_hp(archive->path[0]);
 
 		if (archive->has_password && !xa_check_password(archive))
 			return;
