@@ -5,35 +5,20 @@
  *
  * Copyright (C) 2005  Matthias Clasen <mclasen@redhat.com>
  *
- * Licensed under the Academic Free License version 2.0
- * Or under the following terms:
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * SPDX-License-Identifier: LGPL-2.1-or-later or AFL-2.0
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdio.h>
+//#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <fcntl.h>
 #include <unistd.h>
+#include <errno.h>
 #include <fnmatch.h>
 #include <assert.h>
 
@@ -46,14 +31,14 @@
 #endif
 
 #include <sys/stat.h>
-#include <sys/types.h>
+//#include <sys/types.h>
 
 #include "xdgmimecache.h"
 #include "xdgmimeint.h"
 
-#ifndef MAX
-#define MAX(a,b) ((a) > (b) ? (a) : (b))
-#endif
+//#ifndef MAX
+//#define MAX(a,b) ((a) > (b) ? (a) : (b))
+//#endif
 
 #ifndef	FALSE
 #define	FALSE	(0)
@@ -67,9 +52,13 @@
 #define _O_BINARY 0
 #endif
 
-#ifndef MAP_FAILED
-#define MAP_FAILED ((void *) -1)
-#endif
+//#ifndef MAP_FAILED
+//#define MAP_FAILED ((void *) -1)
+//#endif
+
+//#ifndef S_ISREG
+//#define S_ISREG(m) (((m) & _S_IFMT) == _S_IFREG)
+//#endif
 
 #define MAJOR_VERSION 1
 #define MINOR_VERSION_MIN 1
@@ -87,6 +76,13 @@ struct _XdgMimeCache
 #define GET_UINT16(cache,offset) (ntohs(*(xdg_uint16_t*)((cache) + (offset))))
 #define GET_UINT32(cache,offset) (ntohl(*(xdg_uint32_t*)((cache) + (offset))))
 
+// Validates that it is safe to call GET_UINT32() at
+// cache->buffer[offset + (n * record_size)]. Ensures that offset is aligned to
+// a 4-byte boundary, and that offset+(n*record_size) does not overflow.
+// `record_size` values are known constants and never 0.
+#define OUT_OF_BOUNDS(offset,n,record_size,max) \
+  (((offset) & 0x3) || (offset) > (max) || (n) > ((max) - (offset)) / (record_size))
+
 //XdgMimeCache *
 //_xdg_mime_cache_ref (XdgMimeCache *cache)
 //{
@@ -120,12 +116,15 @@ _xdg_mime_cache_new_from_file (const char *file_name)
   int minor;
 
   /* Open the file and map it into memory */
-  fd = open (file_name, O_RDONLY|_O_BINARY, 0);
+  do {
+    fd = open (file_name, O_RDONLY|_O_BINARY, 0);
+  } while (fd == -1 && errno == EINTR);
 
   if (fd < 0)
     return NULL;
   
-  if (fstat (fd, &st) < 0 || st.st_size < 4)
+  // A valid cache must be at least 40 bytes for the header.
+  if (fstat (fd, &st) < 0 || st.st_size < 40)
     goto done;
 
   buffer = (char *) mmap (NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
@@ -154,6 +153,12 @@ _xdg_mime_cache_new_from_file (const char *file_name)
   if (fd != -1)
     close (fd);
 
+#else /* HAVE_MMAP */
+  cache = (XdgMimeCache *) malloc (sizeof (XdgMimeCache));
+  cache->minor = 0;
+  cache->ref_count = 1;
+  cache->buffer = NULL;
+  cache->size = 0;
 #endif  /* HAVE_MMAP */
 
   return cache;
@@ -171,7 +176,7 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //  xdg_uint32_t data_offset = GET_UINT32 (cache->buffer, offset + 16);
 //  xdg_uint32_t mask_offset = GET_UINT32 (cache->buffer, offset + 20);
 //  
-//  int i, j;
+//  xdg_uint32_t i, j;
 //
 //  for (i = range_start; i < range_start + range_length; i++)
 //    {
@@ -194,7 +199,7 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //	}
 //      else
 //	{
-//	  valid_matchlet = memcmp(cache->buffer + data_offset, data + i, data_length) == 0;
+//	  valid_matchlet = memcmp(cache->buffer + data_offset, (unsigned char *)data + i, data_length) == 0;
 //	}
 //
 //      if (valid_matchlet)
@@ -212,8 +217,10 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //{
 //  xdg_uint32_t n_children = GET_UINT32 (cache->buffer, offset + 24);
 //  xdg_uint32_t child_offset = GET_UINT32 (cache->buffer, offset + 28);
+//  if (OUT_OF_BOUNDS (child_offset, n_children, 32, cache->size))
+//    return FALSE;
 //
-//  int i;
+//  xdg_uint32_t i;
 //  
 //  if (cache_magic_matchlet_compare_to_data (cache, offset, data, len))
 //    {
@@ -238,12 +245,19 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //			     size_t        len, 
 //			     int          *prio)
 //{
-//  xdg_uint32_t priority = GET_UINT32 (cache->buffer, offset);
-//  xdg_uint32_t mimetype_offset = GET_UINT32 (cache->buffer, offset + 4);
-//  xdg_uint32_t n_matchlets = GET_UINT32 (cache->buffer, offset + 8);
-//  xdg_uint32_t matchlet_offset = GET_UINT32 (cache->buffer, offset + 12);
+//  xdg_uint32_t priority, mimetype_offset, n_matchlets, matchlet_offset;
 //
-//  int i;
+//  assert (cache->buffer != NULL);
+//
+//  priority = GET_UINT32 (cache->buffer, offset);
+//  mimetype_offset = GET_UINT32 (cache->buffer, offset + 4);
+//  n_matchlets = GET_UINT32 (cache->buffer, offset + 8);
+//  matchlet_offset = GET_UINT32 (cache->buffer, offset + 12);
+//
+//  if (OUT_OF_BOUNDS (matchlet_offset, n_matchlets, 32, cache->size))
+//    return NULL;
+//
+//  xdg_uint32_t i;
 //
 //  for (i = 0; i < n_matchlets; i++)
 //    {
@@ -263,21 +277,26 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //cache_magic_lookup_data (XdgMimeCache *cache, 
 //			 const void   *data, 
 //			 size_t        len, 
-//			 int          *prio,
-//			 const char   *mime_types[],
-//			 int           n_mime_types)
+//			 int          *prio)
 //{
 //  xdg_uint32_t list_offset;
 //  xdg_uint32_t n_entries;
 //  xdg_uint32_t offset;
 //
-//  int j, n;
+//  xdg_uint32_t j;
+//
+//  assert (cache->buffer != NULL);
 //
 //  *prio = 0;
 //
 //  list_offset = GET_UINT32 (cache->buffer, 24);
+//  if (OUT_OF_BOUNDS (list_offset, 1, 12, cache->size))
+//    return NULL;
+//
 //  n_entries = GET_UINT32 (cache->buffer, list_offset);
 //  offset = GET_UINT32 (cache->buffer, list_offset + 8);
+//  if (OUT_OF_BOUNDS (offset, n_entries, 16, cache->size))
+//    return NULL;
 //  
 //  for (j = 0; j < n_entries; j++)
 //    {
@@ -287,21 +306,6 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //					   data, len, prio);
 //      if (match)
 //	return match;
-//      else
-//	{
-//	  xdg_uint32_t mimetype_offset;
-//	  const char *non_match;
-//	  
-//	  mimetype_offset = GET_UINT32 (cache->buffer, offset + 16 * j + 4);
-//	  non_match = cache->buffer + mimetype_offset;
-//
-//	  for (n = 0; n < n_mime_types; n++)
-//	    {
-//	      if (mime_types[n] && 
-//		  _xdg_mime_mime_type_equal (mime_types[n], non_match))
-//		mime_types[n] = NULL;
-//	    }
-//	}
 //    }
 //
 //  return NULL;
@@ -316,10 +320,21 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 //  for (i = 0; _caches[i]; i++)
 //    {
 //      XdgMimeCache *cache = _caches[i];
-//      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 4);
-//      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      xdg_uint32_t list_offset;
+//      xdg_uint32_t n_entries;
 //      xdg_uint32_t offset;
 //
+//      if (cache->buffer == NULL)
+//        continue;
+//
+//      list_offset = GET_UINT32 (cache->buffer, 4);
+//      if (OUT_OF_BOUNDS (list_offset, 1, 4, cache->size))
+//        continue;
+//
+//      n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      if (OUT_OF_BOUNDS (list_offset + 4, n_entries, 8, cache->size))
+//        continue;
+//
 //      min = 0; 
 //      max = n_entries - 1;
 //      while (max >= min) 
@@ -359,13 +374,26 @@ cache_glob_lookup_literal (const char *file_name,
   const char *ptr;
   int i, min, max, mid, cmp;
 
+  assert (n_mime_types > 0);
+
   for (i = 0; _caches[i]; i++)
     {
       XdgMimeCache *cache = _caches[i];
-      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 12);
-      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+      xdg_uint32_t list_offset;
+      xdg_uint32_t n_entries;
       xdg_uint32_t offset;
 
+      if (cache->buffer == NULL)
+        continue;
+
+      list_offset = GET_UINT32 (cache->buffer, 12);
+      if (OUT_OF_BOUNDS (list_offset, 1, 4, cache->size))
+        continue;
+
+      n_entries = GET_UINT32 (cache->buffer, list_offset);
+      if (OUT_OF_BOUNDS (list_offset + 4, n_entries, 12, cache->size))
+        continue;
+
       min = 0; 
       max = n_entries - 1;
       while (max >= min) 
@@ -410,15 +438,27 @@ cache_glob_lookup_fnmatch (const char *file_name,
   const char *mime_type;
   const char *ptr;
 
-  int i, j, n;
+  int i, n;
+  xdg_uint32_t j;
 
   n = 0;
   for (i = 0; _caches[i]; i++)
     {
       XdgMimeCache *cache = _caches[i];
 
-      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 20);
-      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+      xdg_uint32_t list_offset;
+      xdg_uint32_t n_entries;
+
+      if (cache->buffer == NULL)
+        continue;
+
+      list_offset = GET_UINT32 (cache->buffer, 20);
+      if (OUT_OF_BOUNDS (list_offset, 1, 4, cache->size))
+        continue;
+
+      n_entries = GET_UINT32 (cache->buffer, list_offset);
+      if (OUT_OF_BOUNDS (list_offset + 4, n_entries, 12, cache->size))
+        continue;
 
       for (j = 0; j < n_entries && n < n_mime_types; j++)
 	{
@@ -466,7 +506,8 @@ cache_glob_node_lookup_suffix (XdgMimeCache  *cache,
   int weight;
   int case_sensitive;
 
-  int min, max, mid, n, i;
+  xdg_uint32_t i;
+  int min, max, mid, n;
 
   character = file_name[len - 1];
 
@@ -488,6 +529,8 @@ cache_glob_node_lookup_suffix (XdgMimeCache  *cache,
           n = 0;
           n_children = GET_UINT32 (cache->buffer, offset + 12 * mid + 4);
           child_offset = GET_UINT32 (cache->buffer, offset + 12 * mid + 8);
+          if (OUT_OF_BOUNDS (child_offset, n_children, 12, cache->size))
+            continue;
       
           if (len > 0)
             {
@@ -540,9 +583,21 @@ cache_glob_lookup_suffix (const char *file_name,
     {
       XdgMimeCache *cache = _caches[i];
 
-      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 16);
-      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
-      xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4);
+      xdg_uint32_t list_offset;
+      xdg_uint32_t n_entries;
+      xdg_uint32_t offset;
+
+      if (cache->buffer == NULL)
+        continue;
+
+      list_offset = GET_UINT32 (cache->buffer, 16);
+      if (OUT_OF_BOUNDS (list_offset, 1, 8, cache->size))
+        continue;
+
+      n_entries = GET_UINT32 (cache->buffer, list_offset);
+      offset = GET_UINT32 (cache->buffer, list_offset + 4);
+      if (OUT_OF_BOUNDS (offset, n_entries, 12, cache->size))
+        continue;
 
       n = cache_glob_node_lookup_suffix (cache, 
 					 n_entries, offset, 
@@ -649,7 +704,13 @@ cache_glob_lookup_file_name (const char *file_name,
 //    {
 //      XdgMimeCache *cache = _caches[i];
 //
+//      if (cache->buffer == NULL)
+//        continue;
+//
 //      offset = GET_UINT32 (cache->buffer, 24);
+//      if (OUT_OF_BOUNDS (offset, 1, 8, cache->size))
+//        continue;
+//
 //      max_extent = MAX (max_extent, GET_UINT32 (cache->buffer, offset + 4));
 //    }
 //
@@ -675,8 +736,10 @@ cache_glob_lookup_file_name (const char *file_name,
 //      int prio;
 //      const char *match;
 //
-//      match = cache_magic_lookup_data (cache, data, len, &prio, 
-//				       mime_types, n_mime_types);
+//      if (cache->buffer == NULL)
+//        continue;
+//
+//      match = cache_magic_lookup_data (cache, data, len, &prio);
 //      if (prio > priority)
 //	{
 //	  priority = prio;
@@ -692,12 +755,14 @@ cache_glob_lookup_file_name (const char *file_name,
 //      /* Pick glob-result R where mime_type inherits from R */
 //      for (n = 0; n < n_mime_types; n++)
 //        {
-//          if (mime_types[n] && _xdg_mime_cache_mime_type_subclass(mime_types[n], mime_type))
-//              return mime_types[n];
+//          if (mime_types[n] && _xdg_mime_cache_mime_type_subclass (mime_types[n], mime_type, NULL))
+//            return mime_types[n];
+//        }
+//      if (n == 0)
+//        {
+//          /* No globs: return magic match */
+//          return mime_type;
 //        }
-//
-//      /* Return magic match */
-//      return mime_type;
 //    }
 //
 //  /* Pick first glob result, as fallback */
@@ -785,7 +850,7 @@ cache_glob_lookup_file_name (const char *file_name,
 //					    mime_types, n);
 //
 //  if (!mime_type)
-//    mime_type = _xdg_binary_or_text_fallback(data, bytes_read);
+//    mime_type = _xdg_binary_or_text_fallback (data, bytes_read);
 //
 //  free (data);
 //  fclose (file);
@@ -814,29 +879,41 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 
 //#if 1
 //static int
-//is_super_type (const char *mime)
+//ends_with (const char *str,
+//           const char *suffix)
 //{
 //  int length;
-//  const char *type;
+//  int suffix_length;
 //
-//  length = strlen (mime);
-//  type = &(mime[length - 2]);
+//  length = strlen (str);
+//  suffix_length = strlen (suffix);
+//  if (length < suffix_length)
+//    return 0;
 //
-//  if (strcmp (type, "/*") == 0)
+//  if (strcmp (str + length - suffix_length, suffix) == 0)
 //    return 1;
 //
 //  return 0;
 //}
+
+//static int
+//is_super_type (const char *mime)
+//{
+//  return ends_with (mime, "/*");
+//}
 //#endif
 
 //int
 //_xdg_mime_cache_mime_type_subclass (const char *mime,
-//				    const char *base)
+//				    const char *base,
+//				    const char ***seen)
 //{
-//  const char *umime, *ubase;
+//  const char *umime, *ubase, *parent;
+//  const char **first_seen = NULL, **new_seen;
+//
+//  xdg_uint32_t j;
+//  int i, k, min, max, med, cmp, ret = 0;
 //
-//  int i, j, min, max, med, cmp;
-//  
 //  umime = _xdg_mime_cache_unalias_mime_type (mime);
 //  ubase = _xdg_mime_cache_unalias_mime_type (base);
 //
@@ -858,17 +935,34 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //      strncmp (umime, "text/", 5) == 0)
 //    return 1;
 //
-//  if (strcmp (ubase, "application/octet-stream") == 0)
+//  if (strcmp (ubase, "application/octet-stream") == 0 &&
+//      strncmp (umime, "inode/", 6) != 0)
 //    return 1;
-// 
+//
+//  if (!seen)
+//    {
+//      first_seen = calloc (1, sizeof (char *));
+//      seen = &first_seen;
+//    }
+//
 //  for (i = 0; _caches[i]; i++)
 //    {
 //      XdgMimeCache *cache = _caches[i];
-//      
-//      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 8);
-//      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      xdg_uint32_t list_offset;
+//      xdg_uint32_t n_entries;
 //      xdg_uint32_t offset, n_parents, parent_offset;
 //
+//      if (cache->buffer == NULL)
+//        continue;
+//
+//      list_offset = GET_UINT32 (cache->buffer, 8);
+//      if (OUT_OF_BOUNDS (list_offset, 1, 4, cache->size))
+//        continue;
+//
+//      n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      if (OUT_OF_BOUNDS (list_offset + 4, n_entries, 8, cache->size))
+//        continue;
+//
 //      min = 0; 
 //      max = n_entries - 1;
 //      while (max >= min)
@@ -889,8 +983,27 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //	      for (j = 0; j < n_parents; j++)
 //		{
 //		  parent_offset = GET_UINT32 (cache->buffer, offset + 4 + 4 * j);
-//		  if (_xdg_mime_cache_mime_type_subclass (cache->buffer + parent_offset, ubase))
-//		    return 1;
+//		  parent = cache->buffer + parent_offset;
+//
+//		  /* Detect and avoid buggy circular relationships */
+//		  for (k = 0; (*seen)[k] != NULL; k++)
+//		    if (parent == (*seen)[k])
+//		      goto next_parent;
+//		  new_seen = realloc (*seen, (k + 2) * sizeof (char *));
+//		  if (!new_seen)
+//		    goto done;
+//		  new_seen[k] = parent;
+//		  new_seen[k + 1] = NULL;
+//		  *seen = new_seen;
+//
+//		  if (_xdg_mime_cache_mime_type_subclass (parent, ubase, seen))
+//		    {
+//		      ret = 1;
+//		      goto done;
+//		    }
+//
+//		next_parent:
+//		  continue;
 //		}
 //
 //	      break;
@@ -898,7 +1011,9 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //	}
 //    }
 //
-//  return 0;
+//done:
+//  free (first_seen);
+//  return ret;
 //}
 
 //const char *
@@ -917,7 +1032,8 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //char **
 //_xdg_mime_cache_list_mime_parents (const char *mime)
 //{
-//  int i, j, k, l, p;
+//  int i, l, p;
+//  xdg_uint32_t j, k;
 //  char *all_parents[128]; /* we'll stop at 128 */ 
 //  char **result;
 //
@@ -927,9 +1043,19 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //  for (i = 0; _caches[i]; i++)
 //    {
 //      XdgMimeCache *cache = _caches[i];
-//  
-//      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 8);
-//      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      xdg_uint32_t list_offset;
+//      xdg_uint32_t n_entries;
+//
+//      if (cache->buffer == NULL)
+//        continue;
+//
+//      list_offset = GET_UINT32 (cache->buffer, 8);
+//      if (OUT_OF_BOUNDS (list_offset, 1, 4, cache->size))
+//        continue;
+//
+//      n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      if (OUT_OF_BOUNDS (list_offset + 4, n_entries, 8, cache->size))
+//        continue;
 //
 //      for (j = 0; j < n_entries; j++)
 //	{
@@ -971,7 +1097,7 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //}
 
 //static const char *
-//cache_lookup_icon (const char *mime, int header)
+//cache_lookup_icon (const char *mime, size_t header)
 //{
 //  const char *ptr;
 //  int i, min, max, mid, cmp;
@@ -979,10 +1105,24 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //  for (i = 0; _caches[i]; i++)
 //    {
 //      XdgMimeCache *cache = _caches[i];
-//      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, header);
-//      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      xdg_uint32_t list_offset;
+//      xdg_uint32_t n_entries;
 //      xdg_uint32_t offset;
 //
+//      if (cache->buffer == NULL)
+//        continue;
+//
+//      if (OUT_OF_BOUNDS (header, 1, 4, cache->size))
+//        continue;
+//
+//      list_offset = GET_UINT32 (cache->buffer, header);
+//      if (OUT_OF_BOUNDS (list_offset, 1, 4, cache->size))
+//        continue;
+//
+//      n_entries = GET_UINT32 (cache->buffer, list_offset);
+//      if (OUT_OF_BOUNDS (list_offset + 4, n_entries, 8, cache->size))
+//        continue;
+//
 //      min = 0; 
 //      max = n_entries - 1;
 //      while (max >= min) 
@@ -1029,12 +1169,16 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //  xdg_uint32_t mime_offset;
 //  xdg_uint32_t n_children;
 //  xdg_uint32_t child_offset;
+//  xdg_uint32_t k;
 //  int i;
 //
 //  character = GET_UINT32 (cache->buffer, offset);
 //  mime_offset = GET_UINT32 (cache->buffer, offset + 4);
 //  n_children = GET_UINT32 (cache->buffer, offset + 8);
 //  child_offset = GET_UINT32 (cache->buffer, offset + 12);
+//  if (OUT_OF_BOUNDS (child_offset, n_children, 20, cache->size))
+//    return;
+//
 //  for (i = 0; i < depth; i++)
 //    printf (" ");
 //  printf ("%c", character);
@@ -1043,24 +1187,34 @@ _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 //  printf ("\n");
 //  if (child_offset)
 //  {
-//    for (i = 0; i < n_children; i++)
-//      dump_glob_node (cache, child_offset + 20 * i, depth + 1);
+//    for (k = 0; k < n_children; k++)
+//      dump_glob_node (cache, child_offset + 20 * k, depth + 1);
 //  }
 //}
 
 //void
 //_xdg_mime_cache_glob_dump (void)
 //{
-//  int i, j;
+//  xdg_uint32_t i, j;
 //  for (i = 0; _caches[i]; i++)
 //  {
 //    XdgMimeCache *cache = _caches[i];
 //    xdg_uint32_t list_offset;
 //    xdg_uint32_t n_entries;
 //    xdg_uint32_t offset;
+//
+//    if (cache->buffer == NULL)
+//      continue;
+//
 //    list_offset = GET_UINT32 (cache->buffer, 16);
+//    if (OUT_OF_BOUNDS (list_offset, 1, 8, cache->size))
+//      return;
+//
 //    n_entries = GET_UINT32 (cache->buffer, list_offset);
 //    offset = GET_UINT32 (cache->buffer, list_offset + 4);
+//    if (OUT_OF_BOUNDS (offset, n_entries, 20, cache->size))
+//      return;
+//
 //    for (j = 0; j < n_entries; j++)
 //	    dump_glob_node (cache, offset + 20 * j, 0);
 //  }
