@@ -145,13 +145,10 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	else
 		item[2] = g_strdup("-");
 
-	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item, dir);
 
 	if (entry)
 	{
-		if (dir)
-			entry->is_dir = TRUE;
-
 		entry->is_encrypted = encrypted;
 
 		if (!entry->is_dir)
